<?php

namespace Drupal\pluginreference\Plugin\Field\FieldWidget;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\WidgetBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Plugin implementation of the 'plugin_reference_autocomplete' widget.
 *
 * @FieldWidget(
 *   id = "plugin_reference_autocomplete",
 *   label = @Translation("Autocomplete"),
 *   field_types = {
 *     "plugin_reference"
 *   }
 * )
 */
class PluginReferenceAutocompleteWidget extends WidgetBase {

  use PluginReferenceConfigurationFormTrait;

  /**
   * The Plugin Manager.
   *
   * @var \Drupal\Component\Plugin\PluginManagerInterface
   */
  protected $pluginManager;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    $instance = parent::create($container, $configuration, $plugin_id, $plugin_definition);
    /** @var \Drupal\field\Entity\FieldConfig $field_definition */
    $field_definition = $configuration['field_definition'];
    $target_type = $field_definition->getFieldStorageDefinition()
      ->getSetting('target_type');

    if ($container->has('plugin.manager.' . $target_type)) {
      $instance->pluginManager = $container->get('plugin.manager.' . $target_type);
    }

    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return self::configurationDefaultSettings() + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $elements = parent::settingsForm($form, $form_state);
    return $elements + $this->configurationSettingsForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = parent::settingsSummary();
    return $summary + $this->configurationSettingsSummary();
  }

  /**
   * {@inheritdoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state) {
    $default_value = NULL;

    if (!empty($items[$delta]->plugin_id) && $this->pluginManager->hasDefinition($items[$delta]->plugin_id)) {
      $default_value = $this->pluginManager->getDefinition($items[$delta]->plugin_id);
    }

    $element['plugin_id'] = [
      '#type' => 'plugin_autocomplete',
      '#default_value' => $default_value,
      '#target_type' => $this->getFieldSetting('target_type'),
    ] + $element;

    $element = $this->configurationFormElement($items, $delta, $element, $form, $form_state);

    return $element;
  }

}
